library ieee;
use ieee.std_logic_1164.all;

library unisim;
use unisim.vcomponents.all;

library work;
use work.types.all;
use work.pcie.all;
use work.wb.all;
use work.exzess_pcie_tlpgen_h.all;
use work.exzess_pcie_app_if_h.all;
use work.bmd_design_h.all;
use work.git_version_h.all;
use work.xor_app_h.all;

entity exzess_toplevel is
	generic (
		FAST_TRAIN: boolean := false--;
	);
	port (
		pci_exp_txp: out std_logic;
		pci_exp_txn: out std_logic;
		pci_exp_rxp: in std_logic;
		pci_exp_rxn: in std_logic;

		sys_clk_p: in std_logic;
		sys_clk_n: in std_logic;
		sys_reset_n: in std_logic;

		led: out std_logic_vector(3 downto 0)--;
	);
end entity exzess_toplevel;


architecture exzess_toplevel_arch of exzess_toplevel is
	attribute clock_signal: string;

	signal sys_clk_c: std_logic;
	attribute clock_signal of sys_clk_c: signal is "yes";

	signal trn_i: trn_i_type;
	signal trn_o: trn_o_type;
	signal cfg_i: cfg_i_type;
	signal cfg_o: cfg_o_type;

	signal trn_clk: std_logic;
	attribute clock_signal of trn_clk: signal is "yes";

	signal trn_reset_n: std_logic;

	signal diag_led: std_logic;

	signal sys_reset_n_c: std_logic;

	signal mem_rd_o: mem_rd_o_type;
	signal mem_rd_i: mem_rd_i_type;
	signal mem_rd_rbar_hit: std_logic_vector(6 downto 0);
	signal mem_wr_o: mem_wr_o_type;
	signal mem_wr_i: mem_wr_i_type;
	signal mem_wr_rbar_hit: std_logic_vector(6 downto 0);
	signal cpld_o: cpld_o_type;
	signal cpld_i: cpld_i_type;
	signal cpl_o: cpl_o_type;
	signal cpl_i: cpl_i_type;

	signal tx_mem_rd_i: mem_rd_o_type;
	signal tx_mem_rd_o: mem_rd_i_type;
	signal tx_mem_wr_i: mem_wr_o_type;
	signal tx_mem_wr_o: mem_wr_i_type;
	signal tx_cpld_i: cpld_o_type;
	signal tx_cpld_o: cpld_i_type;

	signal git_v: std_logic_vector(159 downto 0);

	signal mem_request_o: wb_master_o;
	signal mem_request_i: wb_master_i;
	signal app_request_o: wb_master_o;
	signal app_request_i: wb_master_i;
	signal base_address: std_logic_vector(31 downto 0);
begin

refclk_ibuf: IBUFDS port map ( o => sys_clk_c, i => sys_clk_p, ib => sys_clk_n );
sys_reset_n_ibuf: IBUF port map (o => sys_reset_n_c, i => sys_reset_n);

led_0_obuf: OBUF port map ( o => led(0), i => sys_reset_n_c);
led_1_obuf: OBUF port map ( o => led(1), i => trn_reset_n);
led_2_obuf: OBUF port map ( o => led(2), i => trn_i.lnk_up);
led_3_obuf: OBUF port map ( o => led(3), i => diag_led);

git_version_i: git_version port map (git_version => git_v);

cfg_o <= (
	 dwaddr => (others => '0'),
	 err_tlp_cpl_header => (others => '0'),
	 err_posted => '1',
	 interrupt_di => (others => '0'),
	 dsn => git_v(159 downto 96),
	 turnoff_ok => '1',
	 others => '0' -- fixme, dangerous, quite a lot should be handled in a more correct way here...
	 );

xor_app_i: xor_app
port map (
		mem_request_i => mem_request_i,
		mem_request_o => mem_request_o,

		app_request_i => app_request_o,
		app_request_o => app_request_i,

		base_address => base_address,

		clk => trn_clk,
		reset_n => trn_reset_n--,
	 );

exzess_pcie_app_if_i: exzess_pcie_app_if
port map (
	mem_rd_i => mem_rd_o,
	mem_rd_o => mem_rd_i,
	mem_rd_rbar_hit_i => mem_rd_rbar_hit,
	mem_wr_i => mem_wr_o,
	mem_wr_o => mem_wr_i,
	mem_wr_rbar_hit_i => mem_wr_rbar_hit,
	cpld_i => cpld_o,
	cpld_o => cpld_i,
	cpl_i => cpl_o,
	cpl_o => cpl_i,

	tx_mem_rd_i => tx_mem_rd_o,
	tx_mem_rd_o => tx_mem_rd_i,
	tx_mem_wr_i => tx_mem_wr_o,
	tx_mem_wr_o => tx_mem_wr_i,
	tx_cpld_i => tx_cpld_o,
	tx_cpld_o => tx_cpld_i,

	app_request_o => app_request_o,
	app_request_i => app_request_i,
	mem_request_o => mem_request_i,
	mem_request_i => mem_request_o,

	base_address => base_address,

	clk => trn_clk,
	reset_n => trn_reset_n--,
);

-- our application
exzess_pcie_tlpgen_i: exzess_pcie_tlpgen
port map (
	trn_i => trn_i,
	trn_o => trn_o,
	cfg_i => cfg_i,
	-- cfg_o => cfg_o,
	cfg_o => open,

	mem_rd_o => mem_rd_o,
	mem_rd_i => mem_rd_i,
	mem_rd_rbar_hit_o => mem_rd_rbar_hit,
	mem_wr_o => mem_wr_o,
	mem_wr_i => mem_wr_i,
	mem_wr_rbar_hit_o => mem_wr_rbar_hit,
	cpl_o => cpl_o,
	cpl_i => cpl_i,
	cpld_o => cpld_o,
	cpld_i => cpld_i,

	tx_mem_rd_i => tx_mem_rd_i,
	tx_mem_rd_o => tx_mem_rd_o,
	tx_mem_wr_i => tx_mem_wr_i,
	tx_mem_wr_o => tx_mem_wr_o,
	tx_cpld_i => tx_cpld_i,
	tx_cpld_o => tx_cpld_o,

	diag_led => diag_led,

	trn_clk => trn_clk,
	trn_reset_n => trn_reset_n--,
);

-- Xilinx PCIe core
-- all active low ports get inverted because they give me headaches
-- also, wishbone forbids active low (for other, equally good reasons)
bmd_design_i: bmd_design
generic map (FAST_TRAIN => FAST_TRAIN)
port map (
	pci_exp_txp => pci_exp_txp, 
	pci_exp_txn => pci_exp_txn, 
	pci_exp_rxp => pci_exp_rxp, 
	pci_exp_rxn => pci_exp_rxn, 

	"not"(trn_lnk_up_n) =>	trn_i.lnk_up,
	trn_clk =>		trn_clk,
	trn_reset_n =>		trn_reset_n,
	-- Common flow control
	trn_fc_sel =>		trn_o.fc.sel,
	trn_fc_nph =>		trn_i.fc.nph,
	trn_fc_npd =>		trn_i.fc.npd,
	trn_fc_ph =>		trn_i.fc.ph,
	trn_fc_pd =>		trn_i.fc.pd,
	trn_fc_cplh =>		trn_i.fc.cplh,
	trn_fc_cpld =>		trn_i.fc.cpld,
	-- Transaction Tx
	trn_td =>		trn_o.tx.td,
	trn_tsof_n =>		not trn_o.tx.tsof,
	trn_teof_n =>		not trn_o.tx.teof,
	trn_tsrc_rdy_n =>	not trn_o.tx.tsrc_rdy,
	trn_tsrc_dsc_n =>	not trn_o.tx.tsrc_dsc,
	trn_terrfwd_n =>	not trn_o.tx.terrfwd,
	trn_tstr_n =>		not trn_o.tx.tstr,
	trn_tcfg_gnt_n =>	not trn_o.tx.tcfg_gnt,
	"not"(trn_tdst_rdy_n) =>trn_i.tx.tdst_rdy,
	"not"(trn_terr_drop_n)=>trn_i.tx.terr_drop,
	trn_tbuf_av =>		trn_i.tx.tbuf_av,
	"not"(trn_tcfg_req_n) =>trn_i.tx.tcfg_req,
	-- Transaction Rx
	trn_rd =>		trn_i.rx.rd,
	"not"(trn_rsof_n) =>	trn_i.rx.rsof,
	"not"(trn_reof_n) =>	trn_i.rx.reof,
	"not"(trn_rsrc_rdy_n) =>trn_i.rx.rsrc_rdy,
	"not"(trn_rsrc_dsc_n) =>trn_i.rx.rsrc_dsc,
	"not"(trn_rerrfwd_n) =>	trn_i.rx.rerrfwd,
	"not"(trn_rbar_hit_n) =>trn_i.rx.rbar_hit,
	trn_rdst_rdy_n =>	not trn_o.rx.rdst_rdy,
	trn_rnp_ok_n =>		not trn_o.rx.rnp_ok,

	-- Configuration (CFG) Interface
	-- Configuration space access
	cfg_do =>			cfg_i.do,
	"not"(cfg_rd_wr_done_n) =>	cfg_i.rd_wr_done,
	cfg_dwaddr =>			cfg_o.dwaddr,
	cfg_rd_en_n =>			not cfg_o.rd_en,
	-- Error reporting
	cfg_err_ur_n =>			not cfg_o.err_ur,
	cfg_err_cor_n =>		not cfg_o.err_cor,
	cfg_err_ecrc_n =>		not cfg_o.err_ecrc,
	cfg_err_cpl_timeout_n =>	not cfg_o.err_cpl_timeout,
	cfg_err_cpl_abort_n =>		not cfg_o.err_cpl_abort,
	cfg_err_posted_n =>		not cfg_o.err_posted,
	cfg_err_locked_n =>		not cfg_o.err_locked,
	cfg_err_tlp_cpl_header =>	cfg_o.err_tlp_cpl_header,
	"not"(cfg_err_cpl_rdy_n) =>	cfg_i.err_cpl_rdy,
	-- Interrupt generation
	cfg_interrupt_n =>		not cfg_o.interrupt,
	"not"(cfg_interrupt_rdy_n) =>	cfg_i.interrupt_rdy,
	cfg_interrupt_assert_n =>	not cfg_o.interrupt_assert,
	cfg_interrupt_do =>		cfg_i.interrupt_do,
	cfg_interrupt_di =>		cfg_o.interrupt_di,
	cfg_interrupt_mmenable =>	cfg_i.interrupt_mmenable,
	cfg_interrupt_msienable =>	cfg_i.interrupt_msienable,
	-- Power management signaling
	cfg_turnoff_ok_n =>		not cfg_o.turnoff_ok,
	"not"(cfg_to_turnoff_n) =>	cfg_i.to_turnoff,
	cfg_pm_wake_n =>		not cfg_o.pm_wake,
	"not"(cfg_pcie_link_state_n) =>	cfg_i.pcie_link_state,
	cfg_trn_pending_n =>		not cfg_o.trn_pending,
	-- System configuration and status
	cfg_dsn =>			cfg_o.dsn,
	cfg_bus_number =>		cfg_i.bus_number,
	cfg_device_number =>		cfg_i.device_number,
	cfg_function_number =>		cfg_i.function_number,
	cfg_status =>			cfg_i.status,
	cfg_command =>			cfg_i.command,
	cfg_dstatus =>			cfg_i.dstatus,
	cfg_dcommand =>			cfg_i.dcommand,
	cfg_lstatus =>			cfg_i.lstatus,
	cfg_lcommand =>			cfg_i.lcommand,

	-- System (SYS) Interface
	sys_clk =>		sys_clk_c,
	sys_reset_n =>		sys_reset_n_c,
	received_hot_reset =>	open--,
);

end architecture exzess_toplevel_arch;
