library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

library work;
use work.wb.all;

entity xor_app is
	port(
	    	app_request_o: out wb_slave_o;
		app_request_i: in wb_slave_i;

		mem_request_o: out wb_master_o;
		mem_request_i: in wb_master_i;

		base_address: in std_logic_vector(31 downto 0);

		clk: in std_logic;
		reset_n: in std_logic--;
	    );
end entity xor_app;

architecture xor_app_arch of xor_app is
	function valmorphanize(x: in std_logic_vector(31 downto 0)) return std_logic_vector is
	begin
		return x xor X"000F00FF";
	end function valmorphanize;

	signal app_request_o_reg: wb_slave_o;
begin

	app_request_o <= app_request_o_reg;

	xor_stuff: process (clk, reset_n, app_request_i, mem_request_i) is 
		variable timeout: std_logic_vector(5 downto 0);
		variable app_ack_sent: std_logic;
	begin
		if (clk = '1' and clk'event) then
			if (reset_n = '0') then
				app_request_o_reg <= null_wb_slave_o;
				mem_request_o <= null_wb_master_o;
				app_ack_sent := '0';
				timeout := (others => '0');
			else
				if (app_request_i.cyc = '1' and app_request_i.stb = '1') then
					mem_request_o.adr <= std_logic_vector(unsigned(base_address) + unsigned(app_request_i.adr(15 downto 0)));
					if (app_request_i.we = '1') then
						-- write stuff
						if (app_ack_sent = '0' and mem_request_i.ack = '0' and timeout /= "11111") then
							mem_request_o.cyc <= '1';
							mem_request_o.stb <= '1';
							mem_request_o.we <= '1';
							timeout := std_logic_vector(unsigned(timeout) + 1);

							-- address is already calculated and assigned above
							mem_request_o.dat <= valmorphanize(app_request_i.dat);
							app_request_o_reg.ack <= '0';
						else
							if (app_ack_sent = '0') then
								app_ack_sent := '1';
								app_request_o_reg.ack <= '1';
							else
								app_request_o_reg.ack <= '0';
							end if;
							mem_request_o.cyc <= '0';
							mem_request_o.stb <= '0';
							mem_request_o.we <= '0';
						end if;
					else
						-- read stuff
						if (app_ack_sent = '0' and mem_request_i.ack = '0' and timeout /= "11111") then
							-- address is already calculated and assigned above
							mem_request_o.cyc <= '1';
							mem_request_o.stb <= '1';
							timeout := std_logic_vector(unsigned(timeout) + 1);

							mem_request_o.we <= '0';
							app_request_o_reg.ack <= '0';
						else
							if (app_ack_sent = '0') then
								app_ack_sent := '1';
								app_request_o_reg.ack <= '1';
							else
								app_request_o_reg.ack <= '0';
							end if;
							app_request_o_reg.dat <= valmorphanize(mem_request_i.dat);
							mem_request_o.stb <= '0';
							mem_request_o.cyc <= '0';
							mem_request_o.we <= '0';
						end if;
					end if;
				else
					app_ack_sent := '0';
					timeout := (others => '0');
					app_request_o_reg.ack <= '0';
					mem_request_o.cyc <= '0';
					mem_request_o.stb <= '0';
					mem_request_o.we <= '0';
				end if;
			end if;
		end if;
	end process xor_stuff;

end architecture xor_app_arch;
